//
//  ViewController.swift
//  DBSCalendar
//
//  Created by Sreekanth PULICHERLA on 4/2/18.
//  Copyright © 2018 Sreekanth PULICHERLA. All rights reserved.
//

import UIKit
import JTAppleCalendar

protocol getDateDelegate {
    func selectedDate(date: String, selected: Date)
}


class ViewController: UIViewController {
    
    let preDateSelectable: Bool = false
    var delegate: getDateDelegate!
    @IBOutlet weak var calendarCollection: JTAppleCalendarView!
    let outsideMonthColor = UIColor(hex: 0x000000)
    let monthColor = UIColor.black
    let selectedMonthColor = UIColor(hex: 0x3a294b)
    let currentDateSelectedViewColor = UIColor(hex: 0x4e3f5d)
    var selected = Date()
    let formatter = DateFormatter()
    var firstDate : Date?
    var lastDate : Date?
    var frameDimensions : CGFloat = 0.00
    

    override func viewDidLoad() {
        super.viewDidLoad()
        setupCalendarView()
        calendarCollection.register(UINib(nibName: "DateCellHeader", bundle: Bundle.main),
                              forSupplementaryViewOfKind: UICollectionElementKindSectionHeader,
                              withReuseIdentifier: "DateCellHeader")
        calendarCollection.calendarDelegate = self
        calendarCollection.calendarDataSource = self
    }

    
    func setupCalendarView() {
        
        // Setup calendar spacing
        calendarCollection.minimumLineSpacing = 0
        calendarCollection.minimumInteritemSpacing = 0
        calendarCollection.allowsMultipleSelection  = true
        calendarCollection.isRangeSelectionUsed = true
        
        calendarCollection.cellSize = self.view.frame.width/7
        frameDimensions = self.view.frame.width/7

        
        // Setup labels
        calendarCollection.scrollToHeaderForDate(selected)
        calendarCollection.visibleDates { (visibleDates) in
            self.setupViewsOfCalendar(from: visibleDates)
        }
    }
    
    func setupViewsOfCalendar(from visibleDates: DateSegmentInfo) {
        let date = visibleDates.monthDates.first!.date
        
        formatter.dateFormat = "yyyy"
        let year = formatter.string(from: date)
        
        formatter.dateFormat = "MMMM"
        let month = formatter.string(from: date)
        //monthLabel.text = "\(month), \(year)"
    }

    func setDate(date: Date) {
        let formatters = DateFormatter()
        formatters.dateFormat = "dd MMM yyyy"
        self.selected = date
        
    }
    
    func handleCellConfiguration(cell: JTAppleCell?, cellState: CellState) {
        handleCellSelection(view: cell, cellState: cellState)
        handleCellTextColor(view: cell, cellState: cellState)
    }
    
    
    func handleCellSelection(view: JTAppleCell?, cellState: CellState) {
        guard let myCustomCell = view as? DateCelltemplate else {return }
        switch cellState.selectedPosition() {
        case .full:
            myCustomCell.dateView.isHidden = false
            myCustomCell.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMaxXMinYCorner, .layerMinXMaxYCorner, .layerMinXMinYCorner]
            myCustomCell.layer.cornerRadius = myCustomCell.frame.width / 2
        case .left:
            myCustomCell.dateView.isHidden = false
            myCustomCell.layer.cornerRadius = myCustomCell.frame.width / 2
            myCustomCell.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMinXMinYCorner]
        case .right:
            myCustomCell.dateView.isHidden = false
            myCustomCell.layer.cornerRadius = myCustomCell.frame.width / 2
            myCustomCell.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMaxXMinYCorner]
        case .middle:
            myCustomCell.dateView.isHidden = false
            myCustomCell.layer.cornerRadius = 0
        case .none:
            myCustomCell.dateView.isHidden = true
        }
    }
    
    
    func handleCellTextColor(view: JTAppleCell?, cellState: CellState) {
        guard let validCell = view as? DateCelltemplate else { return }
        if cellState.isSelected {
            validCell.dateLabel?.textColor = UIColor.white
        } else {
            
            if cellState.dateBelongsTo == .thisMonth {
                
                validCell.dateLabel?.textColor = getMonthBasedColor(date: cellState.date)
                
                
            } else {
                validCell.dateLabel?.textColor = outsideMonthColor
            }
        }
    }
    
    func getMonthBasedColor (date : Date)-> UIColor{
        
        if (date.dayNumberOfWeek() == 1 || date.dayNumberOfWeek() == 7) {
            return UIColor(hex: 0xA2A2A2)
        } else{
            return UIColor.black
        }
    }
}

extension ViewController: JTAppleCalendarViewDataSource {
    
    func configureCalendar(_ calendar: JTAppleCalendarView) -> ConfigurationParameters {
        
        formatter.dateFormat = "yyyy MM dd"
        formatter.timeZone = Calendar.current.timeZone
        formatter.locale = Calendar.current.locale
        let startDate = Date()
        let endDate = formatter.date(from: "2018 06 30")
        let parameters = ConfigurationParameters(startDate: startDate, endDate: endDate!, numberOfRows: 6, calendar: Calendar.current, generateInDates: InDateCellGeneration.forAllMonths, generateOutDates: OutDateCellGeneration.off, firstDayOfWeek: DaysOfWeek.sunday, hasStrictBoundaries: false)
        //let parameters = ConfigurationParameters(startDate: startDate, endDate: endDate!)
        return parameters
    }
}

extension ViewController: JTAppleCalendarViewDelegate {
    func calendar(_ calendar: JTAppleCalendarView, willDisplay cell: JTAppleCell, forItemAt date: Date, cellState: CellState, indexPath: IndexPath) {
        let cell = cell as! DateCelltemplate
        cell.dateLabel.text = cellState.text
        handleCellConfiguration(cell: cell, cellState: cellState)
    }

    func calendar(_ calendar: JTAppleCalendarView, cellForItemAt date: Date, cellState: CellState, indexPath: IndexPath) -> JTAppleCell {
        let cell = calendar.dequeueReusableCell(withReuseIdentifier: "DateCelltemplate", for: indexPath) as! DateCelltemplate
        self.calendar(calendar, willDisplay: cell, forItemAt: date, cellState: cellState, indexPath: indexPath)
        return cell
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didSelectDate date: Date, cell: JTAppleCell?, cellState: CellState) {
        handleCellConfiguration(cell: cell, cellState: cellState)
        if firstDate != nil {
            if lastDate == nil{
                lastDate = date
            }
            calendarCollection.selectDates(from: firstDate!, to: date,  triggerSelectionDelegate: true, keepSelectionIfMultiSelectionAllowed: true)

        } else {
            firstDate = date
            lastDate = nil
            handleCellConfiguration(cell: cell, cellState: cellState)
            setDate(date: date)
        }
       
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didDeselectDate date: Date, cell: JTAppleCell?, cellState: CellState) {
        handleCellConfiguration(cell: cell, cellState: cellState)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, didScrollToDateSegmentWith visibleDates: DateSegmentInfo) {
        setupViewsOfCalendar(from: visibleDates)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, sectionHeaderSizeFor range: (start: Date, end: Date), belongingTo month: Int) -> CGSize {
        return CGSize(width: 0, height: 40)
    }
    
    func calendarSizeForMonths(_ calendar: JTAppleCalendarView?) -> MonthSize? {
        return MonthSize(defaultSize: 40)
    }
    
    func calendar(_ calendar: JTAppleCalendarView, headerViewForDateRange range: (start: Date, end: Date), at indexPath: IndexPath) -> JTAppleCollectionReusableView {
        let headerCell = calendar.dequeueReusableJTAppleSupplementaryView(withReuseIdentifier: "DateCellHeader", for: indexPath) as! DateCellHeader
        headerCell.dateHeader.text =  range.start.tomorrow.getMonthName()
        return headerCell
    }
}

extension Date {
    var yesterday: Date {
        return Calendar.current.date(byAdding: .day, value: -1, to: noon)!
    }
    var tomorrow: Date {
        return Calendar.current.date(byAdding: .day, value: 1, to: noon)!
    }
    var noon: Date {
        return Calendar.current.date(bySettingHour: 12, minute: 0, second: 0, of: self)!
    }
    var month: Int {
        return Calendar.current.component(.month,  from: self)
    }
    var isLastDayOfMonth: Bool {
        return tomorrow.month != month
    }
    func getMonthName() -> String {
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "MMM"
        let strMonth = dateFormatter.string(from: self)
        return strMonth
    }
    func dayNumberOfWeek() -> Int? {
        return Calendar.current.dateComponents([.weekday], from: self).weekday
    }

}

